/* 
   $Id: sgmemo_main.c,v 1.10 2001/12/25 10:41:54 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgmemo.

   sgmemo is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgmemo is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgmemo; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/X.h>

#include "sgmemo.h"

#include "config.h"

GtkWidget *window;
GtkWidget *notebook;

GtkWidget *main_table;

GtkWidget *memo_table;
GtkWidget *memo_box;
GtkWidget *buttons;

GtkWidget *draw_toggle;

GtkWidget *list;
GtkWidget *list_table;
GtkWidget *list_vscroll;
GtkObject *adjustment;

GtkWidget *memo_text;
GtkWidget *memo_vscroll;

static GnomeUIInfo file_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("Delete"), NULL,
    GTK_SIGNAL_FUNC(press_del_button), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },

  GNOMEUIINFO_SEPARATOR,

  {
    GNOME_APP_UI_ITEM,
    N_("Quit"), NULL,
    GTK_SIGNAL_FUNC(press_quit_button), NULL, NULL,
    GNOME_APP_PIXMAP_STOCK , GNOME_STOCK_MENU_QUIT,
    0, GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu[] = {
  GNOMEUIINFO_MENU_CUT_ITEM((gpointer)cut_command, NULL),
  GNOMEUIINFO_MENU_COPY_ITEM((gpointer)copy_command, NULL),
  GNOMEUIINFO_MENU_PASTE_ITEM((gpointer)paste_command, NULL),
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM((gpointer)selall_text, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo view_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("List"), NULL,
    GTK_SIGNAL_FUNC(press_list_button), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("Write"), NULL,
    GTK_SIGNAL_FUNC(press_edit_button), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_END
};

#if 0
static GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_MENU_ABOUT_ITEM(NULL, NULL), GNOMEUIINFO_END
};
#endif

static GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_SUBTREE(N_("_File"),file_menu),
  GNOMEUIINFO_SUBTREE(N_("_Edit"),edit_menu),
  GNOMEUIINFO_SUBTREE(N_("_View"),view_menu),

#if 0
  GNOMEUIINFO_MENU_HELP_TREE(help_menu),
#endif

  GNOMEUIINFO_END
};


char   *memos[MAX_MEMOS];
int    current_memos = -1;

int    has_copy = 0;
char   *copy_str;
stroke copy_stk[MAX_STROKE];
int    copy_stroke = 0;

stroke stk[MAX_STROKE];
int    undo      = 0;
int    n_stroke  = 0;
int    n_sample  = 0;
int    draw_flag = 0;

char *get_first_line(char *str)
{
  int  len;
  char *ret = NULL;
  char *p = NULL, *q = NULL;

  if(str != NULL){
    len = strlen(str);

    if(len > 0){ 
      q = str;
      do{
	p = strchr(q, '\n');
	if(p && ((p - q) > 0)){
	  ret = (char *) calloc(1, (p - q + 1)*sizeof(char));
	  strncpy(ret, q, p - q);
	  break;
	}else if(!p && strlen(q) > 0){
	  ret = strdup(q);
	  break;
	}
	q = p + 1;
      }while(p);
    }
  }

  return ret;
}

void get_memo(int num)
{
  char *tmp, *p;
  int  len;

  if(memo_text == NULL) return;

  len = gtk_text_get_length(GTK_TEXT(memo_text));
  tmp = gtk_editable_get_chars(GTK_EDITABLE(memo_text), 0, len);
  
  if(tmp != NULL){
    if(memos[num] != NULL) free(memos[num]);
    memos[num] = NULL;

    if(len > 0) memos[num] = strdup(tmp);

    p = get_first_line(memos[num]);

    gtk_clist_set_text(GTK_CLIST(list), num, 1, p);
    if(p) free(p);
  }

  save_memo(num, memos[num]);
}

void set_memo(int num)
{
  if(memos[num]!=NULL){
    gtk_editable_delete_text(GTK_EDITABLE(memo_text), 0, -1);
    gtk_text_insert(GTK_TEXT(memo_text),
		    NULL, NULL,	NULL,
		    memos[num], strlen(memos[num]));
  }else{
    gtk_editable_delete_text(GTK_EDITABLE(memo_text), 0, -1);
  }
}

void 
select_row(GtkWidget *list, gint row, gint column, GdkEventButton *event, gpointer data)
{
  static unsigned int time = 0;

  if((event->time - time) < 200){
    n_stroke = load_stroke(current_memos, stk);
    n_sample = 0;
    set_memo(current_memos);
    
    gtk_editable_set_position(GTK_EDITABLE(memo_text), 0);
    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
  }

  time = event->time;
  current_memos = row;
}

void 
unselect_row(GtkWidget *list, gint row, gint column, GdkEventButton *event, gpointer data)
{
  current_memos = -1;
}

void quit(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit();
}

void close_dialog(GtkWidget *widget, gpointer data)
{
  gtk_grab_remove(widget);
}

void yes_delete(GtkWidget *widget, gpointer data)
{
  if(current_memos < 0) return;

  if(memos[current_memos]){
    free(memos[current_memos]);
    memos[current_memos] = NULL;
    remove_memo(current_memos);
  }

  if(n_stroke > 0){
    remove_stroke(current_memos);
    n_stroke = 0;
  }

  gtk_clist_set_text(GTK_CLIST(list), current_memos, 1, NULL);
  gtk_editable_delete_text(GTK_EDITABLE(memo_text), 0, -1);

  gtk_widget_destroy(GTK_WIDGET(data));
}

void no_delete(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(GTK_WIDGET(data));
}

void
message_btn(char *message, void (*y_func)(), void (*n_func)())
{
  GtkWidget *dialog;
  GtkWidget *d_mesg;
  GtkWidget *y_btn, *n_btn;

  dialog = gtk_dialog_new();

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
		     GTK_SIGNAL_FUNC(close_dialog), NULL);

  gtk_window_set_title(GTK_WINDOW(dialog), NULL);

  d_mesg = gtk_label_new(message);

  gtk_misc_set_padding(GTK_MISC(d_mesg), 10, 10);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     d_mesg, TRUE, TRUE, 0);

  y_btn = gtk_button_new_with_label(_("Yes"));
  n_btn = gtk_button_new_with_label(_("No"));

  gtk_signal_connect(GTK_OBJECT(y_btn), "clicked",
		     GTK_SIGNAL_FUNC(y_func), dialog);
  gtk_signal_connect(GTK_OBJECT(n_btn), "clicked",
		     GTK_SIGNAL_FUNC(n_func), dialog);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     y_btn, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     n_btn, TRUE, TRUE, 0);

  gtk_widget_show(d_mesg);
  gtk_widget_show(y_btn);
  gtk_widget_show(n_btn);
  gtk_widget_show(dialog);
  gtk_grab_add(dialog);
}

void press_list_button(GtkWidget *widget, gpointer data)
{
  if(current_memos < 0) return;

  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) == 1){
    get_memo(current_memos);
    save_stroke(current_memos, stk, n_stroke);
    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
  }
}

void press_edit_button(GtkWidget *widget, gpointer data)
{
  if(current_memos < 0) return;

  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) == 0){
    n_stroke = load_stroke(current_memos, stk);
    n_sample = 0;
    set_memo(current_memos);  
    gtk_editable_set_position(GTK_EDITABLE(memo_text), 0);
    gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
  }
}

void press_del_button(GtkWidget *widget, gpointer data)
{
  message_btn(_("Do you want to delete?"), yes_delete, no_delete);
}

void press_quit_button(GtkWidget *widget, gpointer data)
{
  if(current_memos < 0){
    get_memo(current_memos);
    save_stroke(current_memos, stk, n_stroke);
  }
  gtk_main_quit();
}

#if 0
void button_press(GtkWidget *widget, gpointer data)
{
  switch((gint)data){
  case 0:
    press_list_button(widget, data);
    break;
  case 1:
    press_edit_button(widget, data);
    break;
  case 2:
    press_del_button(widget, data);
    break;
  case 3:
    press_quit_button(widget, data);
    break;
  }
}
#endif

void init_list()
{
  int i;
  char *strs[3];
  char num[256];
  char *list_titles[2];

  list_titles[0] = _("No.");
  list_titles[1] = _("Title");

  list_table = gtk_table_new(2, 2, FALSE);

  list = gtk_clist_new_with_titles(2, list_titles);
  gtk_signal_connect (GTK_OBJECT (list), "select_row", 
		      GTK_SIGNAL_FUNC (select_row), NULL);
  gtk_signal_connect (GTK_OBJECT (list), "unselect_row", 
		      GTK_SIGNAL_FUNC (unselect_row), NULL);

  gtk_clist_column_titles_passive(GTK_CLIST(list));
  gtk_widget_set_usize(list, 1, 1);

  adjustment = gtk_adjustment_new (0.0, 0.0, 1000000.0, 1.0, 25.0, 25.0);
  gtk_clist_set_vadjustment(GTK_CLIST(list), GTK_ADJUSTMENT(adjustment));
  list_vscroll = gtk_vscrollbar_new(GTK_ADJUSTMENT(adjustment));

  gtk_table_attach(GTK_TABLE(list_table), list, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
  
  gtk_table_attach(GTK_TABLE(list_table), list_vscroll, 1, 2, 0, 1,
		   GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);

  gtk_clist_set_column_justification (GTK_CLIST(list), 0, GTK_JUSTIFY_RIGHT);

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), list_table, NULL);

  gtk_widget_show(list);
  gtk_widget_show(list_vscroll);
  gtk_widget_show(list_table);

  strs[0] = num;
  strs[2] = NULL;
  
  for(i = 0; i < MAX_MEMOS; i++){
    sprintf(num, "%d", i + 1);
    memos[i] = load_memo(i);
    strs[1]  = get_first_line(memos[i]);
    gtk_clist_append(GTK_CLIST(list), strs);
    if(strs[1]) free(strs[1]), strs[1] = NULL;
  }
}

gint
text_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  int x,y;
  int start = GTK_TEXT(widget)->first_onscreen_ver_pixel;

  if(undo > 0) undo = 0;

  x = (int) event->x;
  y = (int) event->y + start;

  if((x < 0) || (x > 32767) || (y < 0 ) || (y > 32767))
    return FALSE;

  stk[n_stroke].sample[n_sample].x = (short) x;
  stk[n_stroke].sample[n_sample].y = (short) y;
  n_sample++;
  gtk_text_set_editable(GTK_TEXT(memo_text), 0);

#ifdef DEBUG_PRINT
  printf("sample: %d\n", n_sample);
#endif
  return TRUE;
}

gint
text_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  GdkWindow *win = GTK_TEXT(widget)->text_area;
  GdkGC     *gc;
  int x1,x2,y1,y2;
  int x,y;
  int start = GTK_TEXT(widget)->first_onscreen_ver_pixel;

  x = (int) event->x;
  y = (int) event->y + start;

  if((x < 0) || (x > 32767) || (y < 0 ) || (y > 32767)){
    stk[n_stroke].n_sample = n_sample;
    n_stroke++;
    n_sample = 0;
    return FALSE;
  }

  if(n_sample > 1){
    gc = widget->style->black_gc;
    stk[n_stroke].sample[n_sample].x = (short) x;
    stk[n_stroke].sample[n_sample].y = (short) y;

    x1 = stk[n_stroke].sample[n_sample - 1].x;
    y1 = stk[n_stroke].sample[n_sample - 1].y;
    x2 = stk[n_stroke].sample[n_sample].x;
    y2 = stk[n_stroke].sample[n_sample].y;

    gdk_draw_line(win, gc, x1, y1 - start, x2, y2 - start);

    stk[n_stroke].n_sample = n_sample;
    n_stroke++;
    n_sample = 0;
  }else{
    n_sample = 0;
  }

  gtk_text_set_editable(GTK_TEXT(memo_text), 1);
#ifdef DEBUG_PRINT
  printf("stroke: %d\n", n_stroke);
#endif
  return TRUE;
}

gint
text_motion_notify(GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
  GdkWindow *win = GTK_TEXT(widget)->text_area;
  GdkGC     *gc;
  int       x1,x2,y1,y2;
  int       x,y;
  int start = GTK_TEXT(widget)->first_onscreen_ver_pixel;
  
  x = (int) event->x;
  y = (int) event->y + start;

  if((x < 0) || (x > 32767) || (y < 0 ) || (y > 32767))
    return FALSE;

  if(n_sample > MAX_SAMP){
    stk[n_stroke].n_sample = n_sample;
    n_stroke++;
    n_sample = 0;

    stk[n_stroke].sample[n_sample].x = (short)x;
    stk[n_stroke].sample[n_sample].y = (short)y;
    n_sample++;

  }else  if(n_sample > 0){
    gc = widget->style->black_gc;

    stk[n_stroke].sample[n_sample].x = (short) x;
    stk[n_stroke].sample[n_sample].y = (short) y;

    x1 = stk[n_stroke].sample[n_sample - 1].x;
    y1 = stk[n_stroke].sample[n_sample - 1].y;
    x2 = stk[n_stroke].sample[n_sample].x;
    y2 = stk[n_stroke].sample[n_sample].y;

    gdk_draw_line(win, gc, x1, y1 - start, x2, y2 - start);
    n_sample++;
  }
#ifdef DEBUG_PRINT
  printf("sample: %d\n", n_sample);
#endif
  return TRUE;
}

void draw_stroke(GtkWidget *widget)
{
  GdkWindow *win = GTK_TEXT(widget)->text_area;
  GdkGC     *gc;

  int i, j;
  int x1,x2,y1,y2;
  int start = GTK_TEXT(widget)->first_onscreen_ver_pixel;

  if(!win) return;

  gc = widget->style->black_gc;

  for( i = 0; i < n_stroke; i++){
    for( j = 1; j < stk[i].n_sample; j++){
      x1 = stk[i].sample[j - 1].x;
      y1 = stk[i].sample[j - 1].y;
      x2 = stk[i].sample[j].x;
      y2 = stk[i].sample[j].y;

      gdk_draw_line(win, gc, x1, y1 - start, x2, y2 - start);
    }
  }
}

void text_draw(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  gtk_widget_draw_default(widget);
  draw_stroke(widget);
#ifdef DEBUG_PRINT
  printf(__FUNCTION__"\n");
#endif
}

gint text_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{

#ifdef DEBUG_PRINT
  printf(__FUNCTION__":event->type: %d\n", event->type);
#endif
  switch(event->type){
  case GDK_BUTTON_PRESS:
    if(draw_flag){
      text_button_press(widget, (GdkEventButton *) event, data);
      return TRUE;
    }
    break;
  case  GDK_BUTTON_RELEASE:
    if(draw_flag){
      text_button_release(widget, (GdkEventButton *) event, data);
      return TRUE;
    }
    break;
  case  GDK_MOTION_NOTIFY:
    if(draw_flag){
      text_motion_notify(widget, (GdkEventMotion *) event, data);
      return TRUE;
    }else{
      text_draw(widget, (GdkEventExpose *) event, data);
    }
    break;

  case GDK_EXPOSE:
    gtk_widget_draw(widget, NULL);
    draw_stroke(widget);
    return TRUE;

  default:
#ifdef DEBUG_PRINT
    printf("type: %d\n", event->type);
#endif
    break;
  }

  return FALSE;
}

void scroll_event(GtkObject *object, GdkEvent *event, gpointer data)
{
#ifdef DEBUG_PRINT
  printf(__FUNCTION__": value: %f\n", GTK_ADJUSTMENT(object)->value);
#endif

  text_draw(memo_text, NULL, 0);
}

void text_changed(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  GdkRectangle rect;

  rect.x = 0;
  rect.y = 0;
  rect.width = widget->allocation.width;
  rect.height = widget->allocation.height;
  gtk_widget_draw(widget, &rect);

#ifdef DEBUG_PRINT
  printf(__FUNCTION__"\n");
#endif
}

void undo_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  if(!draw_flag) return;

  if(n_stroke > 0) n_stroke--,undo++;

#ifdef DEBUG_PRINT
  printf(__FUNCTION__":stroke: %d\n", n_stroke);
#endif

  gtk_widget_draw(memo_text, NULL);
}

void redo_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  if(!draw_flag) return;

  if(undo > 0) n_stroke++, undo--;

#ifdef DEBUG_PRINT
  printf(__FUNCTION__":stroke: %d\n", n_stroke);
#endif

  gtk_widget_draw(memo_text, NULL);
}

void copy_command()
{
  int i;
  
  i = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(i == 1) copy_text();
  else       copy_memo();
}

void copy_text()
{
  if(memo_text != NULL)
    gtk_editable_copy_clipboard(GTK_EDITABLE(memo_text));
}

void copy_memo()
{
  if(current_memos < 0) return;

  if(copy_str != NULL) free(copy_str);
  copy_str    = strdup(memos[current_memos]);
  copy_stroke = load_stroke(current_memos, copy_stk);

  has_copy = 1;
}

void cut_command()
{
  int i;

  i = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(i == 1) cut_text();
  else       cut_memo();
}

void cut_text()
{
  if(memo_text != NULL)
    gtk_editable_cut_clipboard(GTK_EDITABLE(memo_text));
}

void cut_memo()
{
  if(current_memos < 0) return;

  copy_memo();

  if(memos[current_memos] != NULL){
    free(memos[current_memos]);
    memos[current_memos] = NULL;
    remove_memo(current_memos);
    n_stroke = 0;
    n_sample = 0;
    remove_stroke(current_memos);
  }
  gtk_clist_set_text(GTK_CLIST(list), current_memos, 1, NULL);
}

void paste_command()
{
  int i;

  i = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(i == 1) paste_text();
  else       paste_memo();
}

void paste_text()
{
  if(memo_text != NULL)
    gtk_editable_paste_clipboard(GTK_EDITABLE(memo_text));
}

void yes_over_write(GtkWidget *widget, gpointer data)
{
  char *p;

  free(memos[current_memos]);
  memos[current_memos] = NULL;
  remove_memo(current_memos);
  n_stroke = 0;
  n_sample = 0;
  remove_stroke(current_memos);
  
  save_memo(current_memos, copy_str);
  memos[current_memos] = strdup(copy_str);
  save_stroke(current_memos, copy_stk, copy_stroke);
  
  p = get_first_line(memos[current_memos]);
  gtk_clist_set_text(GTK_CLIST(list), current_memos, 1, p);
  if(p) free(p);

  gtk_widget_destroy(GTK_WIDGET(data));
}

void no_over_write(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(GTK_WIDGET(data));
}

void paste_memo()
{
  char *p;

  if(current_memos < 0) return;

  if(has_copy){
    if(memos[current_memos] != NULL){
      message_btn(_("Do you want to over write?"),
		  yes_over_write, no_over_write);
    }else{
      memos[current_memos] = strdup(copy_str);
      save_memo(current_memos, copy_str);
      save_stroke(current_memos, copy_stk, copy_stroke);

      p = get_first_line(memos[current_memos]);
      gtk_clist_set_text(GTK_CLIST(list), current_memos, 1, p);
      if(p) free(p);
    }
  }
}

void selall_text()
{
  if(memo_text != NULL)
    gtk_editable_select_region(GTK_EDITABLE(memo_text), 0,
			       gtk_text_get_length(GTK_TEXT(memo_text)));
}

void init_memo()
{
  memo_table = gtk_table_new(2, 2, FALSE);

  memo_text = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(memo_text), TRUE);

  gtk_signal_connect(GTK_OBJECT(memo_text), "event",
		     GTK_SIGNAL_FUNC(text_event), (gpointer)0);  

  gtk_signal_connect(GTK_OBJECT(memo_text), "changed",
		     GTK_SIGNAL_FUNC(text_changed), (gpointer)0);  

  gtk_signal_connect(GTK_OBJECT(memo_text), "draw",
		     GTK_SIGNAL_FUNC(text_draw), (gpointer)0);  

  memo_vscroll = gtk_vscrollbar_new((GTK_TEXT(memo_text))->vadj);

  gtk_signal_connect(GTK_OBJECT((GTK_TEXT(memo_text))->vadj),
		     "value_changed",
		     GTK_SIGNAL_FUNC(scroll_event), (gpointer)0);  

  gtk_table_attach(GTK_TABLE(memo_table), memo_text, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);  
  
  gtk_table_attach(GTK_TABLE(memo_table), memo_vscroll, 1, 2, 0, 1,
		   GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);  

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), memo_table, NULL);

  gtk_widget_show(memo_vscroll);
  gtk_widget_show(memo_text);

  gtk_widget_show(memo_table);
}

void
toggled(GtkWidget *toggle, GdkEvent *event, gpointer data)
{
  draw_flag = GTK_TOGGLE_BUTTON(toggle)->active;
}

GtkWidget *
create_button(GtkWidget *container, char *label, GtkSignalFunc func, int num, int x, int y)
{
  GtkWidget *btn;

  btn = gtk_button_new_with_label(label);

  if(func != NULL)
#if 1 /* kon  2001/Aug/31 */
    gtk_signal_connect(GTK_OBJECT(btn), "clicked",
		       GTK_SIGNAL_FUNC(func), (gpointer)num);
#else
    gtk_signal_connect(GTK_OBJECT(btn), "button_press_event",
		       GTK_SIGNAL_FUNC(func), (gpointer)num);
#endif

  gtk_table_attach(GTK_TABLE(container), btn,
		   x, x + 1, y, y + 1,
		   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
  
  gtk_widget_show(btn);

  return btn;
}

void init_buttons()
{
  buttons =  gtk_table_new(2, 5, FALSE);
  gtk_table_attach(GTK_TABLE(main_table), buttons, 0, 1, 1, 2,
		   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);  

  draw_toggle = gtk_toggle_button_new_with_label(_("Hand"));
  gtk_signal_connect(GTK_OBJECT(draw_toggle), "toggled",
		     GTK_SIGNAL_FUNC(toggled), 0);

  gtk_table_attach(GTK_TABLE(buttons), draw_toggle,
		   0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
  gtk_widget_show(draw_toggle);

  create_button(buttons, _("Undo"), undo_button_press, 0, 1, 0);
  create_button(buttons, _("Redo"), redo_button_press, 0, 2, 0);

  create_button(buttons, _("List"),   press_list_button, 0, 0, 1);
  create_button(buttons, _("Write"),   press_edit_button, 1, 1, 1);
  create_button(buttons, _("Delete"), press_del_button,  2, 2, 1);

#if 0
  create_button(buttons, _("Quit"),   press_quit_btn, 3, 3, 1);
#endif

  gtk_widget_show(buttons);
}

void
init_notebook()
{
  notebook = gtk_notebook_new();

  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
  gtk_table_attach(GTK_TABLE(main_table), notebook, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);  
  gtk_widget_show(notebook);
}

int main(int argc, char **argv)
{
  GtkWidget *app;
  int status;

  status = SGCheckDinvoke(argv[0]);
  if(status > 0) return -1;

  bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain(PACKAGE);

  gtk_set_locale();

  gnome_init (PACKAGE, VERSION, argc, argv);
  app = gnome_app_new(PACKAGE, _("sgmemo"));

  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (quit), NULL);

  gnome_app_create_menus(GNOME_APP(app), main_menu);

  main_table = gtk_table_new(2, 2, FALSE);
  gnome_app_set_contents(GNOME_APP(app), main_table);

  init_notebook();
  init_list();
  init_memo();
  init_buttons();

  gtk_widget_show(main_table);
  gtk_widget_show(app);
  
  set_memo(0);  
  n_stroke = load_stroke(0, stk);
  n_sample = 0;

  gtk_main();

  return 1;
}
